@@ -92,6 +92,9 @@ BEGIN
     )],
      'net' => [qw(
       &argonaut_get_mac
+    )],
+     'utils' => [qw(
+      &argonaut_check_time_frames
     )],
      'config' => [qw(
       &argonaut_read_config
@@ -653,7 +656,7 @@ sub argonaut_read_ldap_config {
       } else {
         if (($mesg->entries)[0]->get_value("$value")) {
           $config->{"$key"} = ($mesg->entries)[0]->get_value("$value");
-        } else {
+        } elsif (not defined $config->{"$key"}) {
           $config->{"$key"} = "";
         }
       }
@@ -682,71 +685,85 @@ sub argonaut_get_generic_settings {
     $filter = "($filter)";
   }
 
+  my @attrs = ('dn', 'ipHostNumber', 'macAddress', 'gotoMode', 'fdMode', 'argonautDeploymentTimeframe');
+  for my $param (values(%{$params})) {
+    if (ref $param eq ref []) {
+      push @attrs, $param->[0]
+    } else {
+      push @attrs, $param;
+    }
+  }
   my $mesg = $ldap->search( # perform a search
-            base   => $ldap_base,
-            filter => "(&(objectClass=$objectClass)$filter)"
-            );
+    base    => $ldap_base,
+    filter  => "(&(objectClass=$objectClass)$filter)",
+    attrs   => \@attrs
+  );
 
   my $settings = {
   };
 
-  if(scalar($mesg->entries)==1) {
-    $settings->{'dn'}   = ($mesg->entries)[0]->dn();
-    $settings->{'mac'}  = ($mesg->entries)[0]->get_value("macAddress");
-    $settings->{'ip'}   = ($mesg->entries)[0]->get_value("ipHostNumber");
-    if (($mesg->entries)[0]->exists('fdMode')) {
-      $settings->{'locked'} = ($mesg->entries)[0]->get_value("fdMode") eq 'locked';
-    } elsif (($mesg->entries)[0]->exists('gotoMode')) {
-      $settings->{'locked'} = ($mesg->entries)[0]->get_value("gotoMode") eq 'locked';
-    } else {
-      $settings->{'locked'} = 0;
-    }
-    while (my ($key,$value) = each(%{$params})) {
-      if (ref $value eq ref []) {
-        $settings->{"$key"} = ($mesg->entries)[0]->get_value(@$value);
-      } else {
-        if (($mesg->entries)[0]->get_value("$value")) {
-          $settings->{"$key"} = ($mesg->entries)[0]->get_value("$value");
-        } else {
-          $settings->{"$key"} = "";
-        }
-      }
-    }
-    return $settings;
-  } elsif(scalar($mesg->entries)==0) {
+  my $foundOC = 0;
+  if (scalar($mesg->entries) > 1) {
+    die "Several computers matches $filter.$die_endl";
+  } elsif (scalar($mesg->entries) == 0) {
     unless ($inheritance) {
       die "This computer ($filter) is not configured in LDAP to run this module (missing service $objectClass).$die_endl";
     }
-    $mesg = $ldap->search( # perform a search
-              base   => $ldap_base,
-              filter => $filter,
-              attrs => [ 'dn', 'macAddress', 'gotoMode', 'fdMode' ]
-              );
-    if (scalar($mesg->entries)>1) {
+    $mesg = $ldap->search( # Get the system object
+      base    => $ldap_base,
+      filter  => $filter,
+      attrs   => \@attrs
+    );
+    if (scalar($mesg->entries) > 1) {
       die "Several computers matches $filter.$die_endl";
-    } elsif (scalar($mesg->entries)<1) {
+    } elsif (scalar($mesg->entries) < 1) {
       die "There is no computer matching $filter.$die_endl";
     }
-    $settings->{'dn'}   = ($mesg->entries)[0]->dn();
-    $settings->{'mac'}  = ($mesg->entries)[0]->get_value("macAddress");
-    $settings->{'ip'}   = ($mesg->entries)[0]->get_value("ipHostNumber");
-    if (($mesg->entries)[0]->exists('fdMode')) {
-      $settings->{'locked'} = ($mesg->entries)[0]->get_value("fdMode") eq 'locked';
-    } elsif (($mesg->entries)[0]->exists('gotoMode')) {
-      $settings->{'locked'} = ($mesg->entries)[0]->get_value("gotoMode") eq 'locked';
-    } else {
-      $settings->{'locked'} = 0;
+  } else {
+    $foundOC = 1;
+    while (my ($key,$value) = each(%{$params})) {
+      if (ref $value eq ref []) {
+        $settings->{"$key"} = ($mesg->entries)[0]->get_value(@$value);
+      } elsif (($mesg->entries)[0]->get_value("$value")) {
+        $settings->{"$key"} = ($mesg->entries)[0]->get_value("$value");
+      } else {
+        $settings->{"$key"} = "";
+      }
     }
-    my $dn = ($mesg->entries)[0]->dn();
-    my $mesg = $ldap->search( # perform a search
-      base   => $ldap_base,
-      filter => "(&(objectClass=$objectClass)(member=$dn))",
-      attrs => [values(%{$params})]
-    );
-    if(scalar($mesg->entries)==1) {
+  }
+
+  $settings->{'dn'}   = ($mesg->entries)[0]->dn();
+  $settings->{'mac'}  = ($mesg->entries)[0]->get_value("macAddress");
+  $settings->{'ip'}   = ($mesg->entries)[0]->get_value("ipHostNumber");
+  if (($mesg->entries)[0]->exists('fdMode')) {
+    $settings->{'locked'} = ($mesg->entries)[0]->get_value("fdMode") eq 'locked';
+  } elsif (($mesg->entries)[0]->exists('gotoMode')) {
+    $settings->{'locked'} = ($mesg->entries)[0]->get_value("gotoMode") eq 'locked';
+  } else {
+    $settings->{'locked'} = 0;
+  }
+  $settings->{'timeframes'} = ($mesg->entries)[0]->get_value('argonautDeploymentTimeframe', asref => 1);
+
+  my $dn = ($mesg->entries)[0]->dn();
+  my $mesgGroup = $ldap->search( # Get the group object
+    base   => $ldap_base,
+    filter => "(&(objectClass=$objectClass)(member=$dn))",
+    attrs  => \@attrs
+  );
+
+  if (scalar($mesgGroup->entries) == 1) {
+    if (not defined $settings->{'timeframes'}) {
+      $settings->{'timeframes'} = ($mesgGroup->entries)[0]->get_value('argonautDeploymentTimeframe', asref => 1);
+    }
+  }
+
+  if (not $foundOC) {
+    if (scalar($mesgGroup->entries) == 1) {
       while (my ($key,$value) = each(%{$params})) {
-        if (($mesg->entries)[0]->get_value("$value")) {
-          $settings->{"$key"} = ($mesg->entries)[0]->get_value("$value");
+        if (ref $value eq ref []) {
+          $settings->{"$key"} = ($mesgGroup->entries)[0]->get_value(@$value);
+        } elsif (($mesgGroup->entries)[0]->get_value("$value")) {
+          $settings->{"$key"} = ($mesgGroup->entries)[0]->get_value("$value");
         } else {
           $settings->{"$key"} = "";
         }
@@ -755,9 +772,26 @@ sub argonaut_get_generic_settings {
     } else {
       die "This computer ($filter) is not configured in LDAP to run this module (missing service $objectClass).$die_endl";
     }
-  } else {
-    die "Several computers matches $filter.$die_endl";
   }
+
+  if ($inheritance == 2) {
+    # Get group values on top of normal ones
+    if (scalar($mesgGroup->entries) == 1) {
+      $settings->{"group"} = {};
+      while (my ($key,$value) = each(%{$params})) {
+        if (ref $value eq ref []) {
+          $settings->{"group"}->{"$key"} = ($mesgGroup->entries)[0]->get_value(@$value);
+        } elsif (($mesgGroup->entries)[0]->get_value("$value")) {
+          $settings->{"group"}->{"$key"} = ($mesgGroup->entries)[0]->get_value("$value");
+        } else {
+          $settings->{"group"}->{"$key"} = "";
+        }
+      }
+      return $settings;
+    }
+  }
+
+  return $settings;
 }
 
 #------------------------------------------------------------------------------
@@ -783,7 +817,8 @@ sub argonaut_get_server_settings {
       'delete_finished_tasks' => "argonautDeleteFinished",
       'fetch_packages'        => "argonautFetchPackages",
       'interface'             => "argonautWakeOnLanInterface",
-      'logdir'                => "argonautLogDir"
+      'logdir'                => "argonautLogDir",
+      'timeout'               => "argonautTimeout"
     },
     $config,$ip
   );
@@ -839,6 +874,42 @@ sub argonaut_get_fuse_settings {
   );
 }
 
+#------------------------------------------------------------------------------
+# check if we are in an authorized time frame - Returns true if we are
+#
+sub argonaut_check_time_frames {
+  my ($settings) = @_;
+  if (not defined $settings->{'timeframes'}) {
+    return 1;
+  }
+  foreach my $frame (@{$settings->{'timeframes'}}) {
+    if ($frame =~ m/(\d\d)(\d\d)-(\d\d)(\d\d)/) {
+      my ($sec,$min,$hour) = gmtime(time());
+      my $begin = ($1 * 60) + $2;
+      my $end   = ($3 * 60) + $4;
+      my $now   = ($hour * 60) + $min;
+      if ($begin > $end) {
+        # Frame over midnight
+        $end += 24 * 60;
+        if ($now < $begin) {
+          $now += 24 * 60;
+        }
+      }
+      if ($now < $begin) {
+        # Too soon
+        next;
+      } elsif ($now > $end) {
+        # Too late
+        next;
+      }
+      return 1;
+    } else {
+      die "Invalid value in time frames: $frame".$die_endl;
+    }
+  }
+  return 0;
+}
+
 #------------------------------------------------------------------------------
 #
 # $search_result = Net::LDAP::Serach
