@@ -237,10 +237,11 @@ sub parse_package_list_debian {
       }
       my $packages = $distributions->{$repo->{'release'}."/$section"};
       foreach my $arch (@{$repo->{'archs'}}) {
-          my $packages_file = "$packages_folder/$localuri/dists/".$repo->{'release'}."/$section/binary-$arch/Packages";
-          open (PACKAGES, "<$packages_file") or next;
+          my $packages_filepath = "$packages_folder/$localuri/dists/".$repo->{'release'}."/$section/binary-$arch/Packages";
+          my $packages_file;
+          open ($packages_file, q{<}, $packages_filepath) or next;
           my $parsed = {};
-          while (<PACKAGES>) {
+          while (<$packages_file>) {
               if (/^$/) {
                   # Empty line means this package info lines are over
                   $$package_indice++;
@@ -287,7 +288,7 @@ sub parse_package_list_debian {
                           }
                       }
                       if($match == 0) {
-                          while(<PACKAGES>) {
+                          while(<$packages_file>) {
                               if (/^$/) {
                                   last;
                               }
@@ -301,7 +302,7 @@ sub parse_package_list_debian {
                           $parsed->{'DESCRIPTION'} = encode_base64($value);
                       } elsif ((uc($key) eq 'PACKAGE') && (defined $packages->{$value}) && !(grep {uc($_) eq 'VERSION'} @{$attrs})) {
                           # We already have the info on this package and version was not asked, skip to next one.
-                          while(<PACKAGES>) {
+                          while(<$packages_file>) {
                               if (/^$/) {
                                   last;
                               }
@@ -315,7 +316,7 @@ sub parse_package_list_debian {
                               push @versions, $value;
                           }
                           $packages->{$parsed->{'PACKAGE'}}->{'VERSION'} = join(',',@versions);
-                          while(<PACKAGES>) {
+                          while(<$packages_file>) {
                               if (/^$/) {
                                   last;
                               }
@@ -336,7 +337,7 @@ sub parse_package_list_debian {
                   }
               }
           }
-          close(PACKAGES);
+          close($packages_file);
       }
       if((defined $to) && ($$package_indice>$to)) {
           last;
@@ -470,6 +471,7 @@ Extract templates from packages.
 =cut
 sub cleanup_and_extract {
     my ($servdir,$distribs) = @_;
+    my $file;
 
     my $tmpdir = "/tmp";
     mkpath($tmpdir);
@@ -486,14 +488,14 @@ sub cleanup_and_extract {
             if( -f "$tmpdir/DEBIAN/templates" ) {
                 my $tmpl = encode_base64(file("$tmpdir/DEBIAN/templates")->slurp());
 
-                open (FILE, ">$outdir/".$package->{'PACKAGE'}) or die "cannot open file";
-                print FILE $tmpl;
-                close(FILE);
-                unlink("$tmpdir/DEBIAN/templates");
+                open ($file,q{>},"$outdir/".$package->{'PACKAGE'}) or die "cannot open file";
+                print $file $tmpl;
+                close ($file);
+                unlink ("$tmpdir/DEBIAN/templates");
             } else {
-                open (FILE, ">$outdir/".$package->{'PACKAGE'}.'-NOTEMPLATE') or die "cannot open file";
-                print FILE "1\n";
-                close(FILE);
+                open ($file,q{>},"$outdir/".$package->{'PACKAGE'}.'-NOTEMPLATE') or die "cannot open file";
+                print $file "1\n";
+                close ($file);
             }
         }
     }
