@@ -23,11 +23,15 @@
 #######################################################################
 
 package XML::SAX::RPMRepomdHandler;
+
+use strict;
+use warnings;
+
 use base qw(XML::SAX::Base);
 
   sub new {
     my $class = shift;
-    $self = {
+    my $self = {
       'type' => shift || 'primary',
     };
     return bless $self, $class;
