@@ -44,7 +44,7 @@ my $NAMEDCHECKCONF = 'named-checkconf';
 # get ldap2zone settings
 #
 sub argonaut_get_ldap2zone_settings {
-  return argonaut_get_generic_settings(
+  my $settings = argonaut_get_generic_settings(
     'argonautDNSConfig',
     {
       'binddir'       => 'argonautLdap2zoneBindDir',
@@ -62,6 +62,10 @@ sub argonaut_get_ldap2zone_settings {
     },
     @_
   );
+  if (not defined $settings->{'slavefiles'}) {
+    $settings->{'slavefiles'} = [];
+  }
+  return $settings;
 }
 
 =item argonaut_ldap2zone
@@ -132,16 +136,23 @@ sub argonaut_ldap2zone
   } elsif ($slavefiles)  {
     print "Updating all slave files\n" if $verbose;
     my @zones = @{$settings->{'slavefiles'}};
-    my $all_reverse_zones = [];
+    my @all_reverse_zones = ();
     foreach (@zones) {
       my ($zoneName, $masterline, $reverse) = split /\|/, $_, 3;
       print "Updating slave $zoneName\n" if $verbose;
+      create_slave_namedconf($zoneName,$masterline,$BIND_DIR,$BIND_CACHE_DIR,$output_BIND_DIR,$verbose);
       my $zonedn = zonesearch($ldap,$ldap_base,$zoneName,$verbose);
-      my $reverse_zones = [];
       if ($zonedn and ($reverse ne 'noreverse')) {
-        $reverse_zones = get_reverse_zones($ldap,$ldap_base,$zonedn);
+        my $reverse_zones = get_reverse_zones($ldap,$ldap_base,$zonedn);
+        foreach my $reverse_zone (@$reverse_zones) {
+          if (grep {$_ eq $reverse_zone} @all_reverse_zones) {
+            # this avoids treating twice the same reverse zone
+            next;
+          }
+          push @all_reverse_zones, $reverse_zone;
+          create_slave_namedconf($reverse_zone,$masterline,$BIND_DIR,$BIND_CACHE_DIR,$output_BIND_DIR,$verbose);
+        }
       }
-      create_slave_namedconf($zoneName,$masterline,$reverse_zones,$BIND_DIR,$BIND_CACHE_DIR,$output_BIND_DIR,$verbose,$all_reverse_zones);
     }
   } else {
     if (substr($zone,-1) ne ".") { # If the end point is not there, add it
@@ -242,23 +253,27 @@ sub zoneparse
       }
     }
     my $soa = $entry->get_value("sOARecord");
-    if($soa) {
+    if ($soa) {
       my $soa_record = $zonefile->soa();
       my (@soa_fields) = split(' ',$soa);
-      $soa_record->{'primary'}  = $soa_fields[0];
-      $soa_record->{'email'}    = $soa_fields[1];
-      $soa_record->{'serial'}   = $soa_fields[2];
-      $soa_record->{'refresh'}  = $soa_fields[3];
-      $soa_record->{'retry'}    = $soa_fields[4];
-      $soa_record->{'expire'}   = $soa_fields[5];
-      $soa_record->{'minimumTTL'}  = $soa_fields[6];
+      if ((not defined $soa_record->{'serial'}) or ($soa_fields[2] > $soa_record->{'serial'})) {
+        $soa_record->{'primary'}  = $soa_fields[0];
+        $soa_record->{'email'}    = $soa_fields[1];
+        $soa_record->{'serial'}   = $soa_fields[2];
+        $soa_record->{'refresh'}  = $soa_fields[3];
+        $soa_record->{'retry'}    = $soa_fields[4];
+        $soa_record->{'expire'}   = $soa_fields[5];
+        $soa_record->{'minimumTTL'}  = $soa_fields[6];
 
-      $soa_record->{'class'}    = $class;
-      $soa_record->{'ttl'}      = $TTL;
-      $soa_record->{'origin'}   = $name;
-      $soa_record->{'ORIGIN'}   = $zone;
-      print "Added record SOA $name $class $soa $TTL\n" if $verbose;
-      $dn = $entry->dn();
+        $soa_record->{'class'}    = $class;
+        $soa_record->{'ttl'}      = $TTL;
+        $soa_record->{'origin'}   = $name;
+        $soa_record->{'ORIGIN'}   = $zone;
+        print "Added record SOA $name $class $soa $TTL\n" if $verbose;
+        $dn = $entry->dn();
+      } else {
+        print "Ignored SOA $name $class $soa $TTL (lower serial)\n" if $verbose;
+      }
     }
   }
 
@@ -481,7 +496,7 @@ Returns :
 =cut
 sub create_slave_namedconf
 {
-  my($zone,$masterline,$reverse_zones,$BIND_DIR,$BIND_CACHE_DIR,$output_BIND_DIR,$verbose,$all_reverse_zones) = @_;
+  my($zone,$masterline,$BIND_DIR,$BIND_CACHE_DIR,$output_BIND_DIR,$verbose) = @_;
 
   if (substr($masterline,-1) ne ";") {
     # If the end semi-colon is not there, add it
@@ -500,21 +515,6 @@ zone "$zone" {
   file "$BIND_CACHE_DIR/db.$zone";
 };
 EOF
-  foreach my $reverse_zone (@$reverse_zones) {
-    if (grep {$_ eq $reverse_zone} @$all_reverse_zones) {
-      # this avoids having twice the same reverse zones in slave conf
-      next;
-    }
-    push @$all_reverse_zones, $reverse_zone;
-    print "Writing reverse zone '$reverse_zone'\n" if $verbose;
-    print $namedfile <<EOF;
-zone "$reverse_zone" {
-  type slave;
-  masters {$masterline};
-  file "$BIND_CACHE_DIR/db.$reverse_zone";
-};
-EOF
-  }
   close $namedfile;
 }
 
